/**
  Tiny OpenGL/GLUT particle system example program
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2009-01-21 (Public Domain)
*/
#include "GL/glut.h" /* windows, mice, etc.  http://freeglut.sourceforge.net */
#include <stdlib.h> /* for "rand" */
#include <stdio.h> /* for printf */
#include "osl/vec4.h" /* 3D vector class */
#include <vector> /* std::vector container */

/* Random Float: Return a random number between 0 and 1 */
float rf(void) { return (rand()&0xffff)*(1.0/0xffff); }
/* Random Vector: return a random vector in 3D, all axes between 0 and 1 */
vec3 rv(void) {return vec3(rf(),rf(),rf());}

float rotate_x=0, rotate_y=0;

/* Return the wall clock time, in seconds */
double oglTimer(void) {return 0.001*glutGet(GLUT_ELAPSED_TIME);}

	class particle {
	public:
		vec3 P; /* position */
		vec3 V; /* velocity */
		vec3 C; /* color */
	};

void display(void) /* Draw everything in our window */
{
	static double elapsed=0.0; /* last elapsed time (seconds) */
	glFinish(); double start_t=oglTimer();
	
	glClearColor(0.3,0.5,0.7,1.0); /* clear to a nice blue color */
	glClear(GL_COLOR_BUFFER_BIT);
	
	glRotatef(rotate_x,0,1,0);
	glRotatef(rotate_y,1,0,0);
	
	glPointSize(4.3);
	
	static std::vector<particle> pts;
	if (pts.size()==0) { /* First time: initialize points */
		srand(1);
		for (int i=0;i<100000;i++) {
			particle p;
			p.P=2.0*rv()-vec3(1.0); 
			p.V=0.3*(rv()-vec3(0.5)); 
			p.C=rv();
			pts.push_back(p);
		}
	}
	
	// This keeps track of one particle's Y position 
	static std::vector<float> history;
	
	float dt=0.1;
	glBegin(GL_POINTS);
	int ps=pts.size();
	for (int i=0;i<ps;i++) {
		particle &p=pts[i];
	
	/* Apply physics */
		vec3 A(0.0,-1.0,0.0); /* gravity acelleration */
		p.P+=dt*p.V; /* move position by velocity */
		p.V+=dt*A; /* accelerate particles downwards */
		p.V=p.V*0.98; /* artificial drag */
		
	/* Enforce boundary conditions */
		float floor=-1.0;
		if (p.P.y<floor) { /* we're below the floor */
			p.V.y=-p.V.y; /* bounce off the floor */
		}
	
	/* Draw the particle */
		glColor3fv(p.C);
		glVertex3fv(p.P);
		
		if (i==98) history.push_back(p.P.y);
	}
	glEnd();
	
	// Plot the history buffer on top of everything else
	glColor3f(1,1,1);
	glBegin(GL_LINE_STRIP);
	for (unsigned int h=0;h<history.size();h++) {
		glVertex2f(h*(2.0/history.size())-1.0,history[h]);
	}
	glEnd();
	
	glutSwapBuffers(); /* display everything we've rendered so far */
	glutPostRedisplay(); /* continual animation */
	
	// Print frames per second
	glFinish(); elapsed=oglTimer()-start_t; /* stop the stopwatch */
	static double last_print=0.0; /* Periodically print out the time */
	if (start_t>last_print+0.5) {
		printf("%.1f frames/sec, %.2f ms/frame\n",
			1.0/elapsed,1000.0*elapsed);
		last_print=start_t;
	}
}

void motion(int x, int y) { 
	float scale=0.01; /* degrees rotation/frame per mouse pixel motion */
	rotate_x=scale*(x-glutGet(GLUT_WINDOW_WIDTH)/2); 
	rotate_y=scale*(y-glutGet(GLUT_WINDOW_HEIGHT)/2); 
}

int main(int argc,char **argv) {
	glutInit(&argc,argv);  /* set up GLUT */
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutCreateWindow("Particle Demo"); /* make a window */
	glutDisplayFunc(display); /* here's how to draw the window */
	glutMotionFunc(motion); /* here's how to draw the window */
	glutMainLoop(); /* start drawing */
}
